set.seed(1)
setwd('~/Documents/Travail/monitorat/2014/M2IM/random_forest/Code/')

# Charge les données
data(airquality)
n = nrow(airquality)
airquality = airquality[sample(1:n),]
n.train = floor(0.8* n)
airquality.train = airquality[1:n.train,]
airquality.test = airquality[-(1:n.train),]

rmse = function(x) {sqrt(mean(x^2, na.rm = TRUE))}
# Arbre de régression
library("tree")
ozone.tree <- tree(Ozone ~., data=airquality.train)
summary(ozone.tree)
plot(ozone.tree)
text(ozone.tree)
pred <- predict(ozone.tree, airquality.test)
rmse(pred - airquality.test$Ozone)

# Arbre trop grand --> surapprentissage
ozone.tree1 <- tree(Ozone ~., data=airquality.train, 
                    control = tree.control(nobs = n.train, mindev = 0, minsize = 2))
summary(ozone.tree1)
plot(ozone.tree1)
pred <- predict(ozone.tree1, airquality.test)
rmse(pred - airquality.test$Ozone)

# Élagage
ozone.tree2 <- prune.tree(ozone.tree1, best = 8)
pred <- predict(ozone.tree2, airquality.test)
rmse(pred - airquality.test$Ozone)

# À quel point faut-il élaguer ?
err = NULL
nnodes = seq(2,82,by = 3)
for (i in nnodes) {
  T = prune.tree(ozone.tree1, best = i)
  pred <- predict(T, airquality.test)
  err = c(err,rmse(pred - airquality.test$Ozone))
}
plot(nnodes, err, type='l')

# Déterminer la taille optimale de l'arbre par validation croisée
plot(cv.tree(ozone.tree1, K = 10))

# Forêts aléatoires
library(randomForest)
ozone.rf <- randomForest(Ozone ~ ., data=airquality.train, 
                         mtry=3, importance=TRUE, na.action=na.omit)
print(ozone.rf)
plot(sqrt(ozone.rf$mse), type = 'l', xlab="nombre d'arbres", ylab = "rmse OOB")

pred <- predict(ozone.rf, airquality.test)
rmse(pred-airquality.test$Ozone)

print(ozone.rf)
varImpPlot(ozone.rf, type = 1, main = "Importance des variables pour la prévision de qualité de l'air")

