rm(list=ls())
set.seed(1000)
setwd('~/Documents/Travail/monitorat/2014/M2IM/random_forest/Code/')

# Charge les données
library("ElemStatLearn")
data(spam)
source('spam_names.r')
n = nrow(spam)
spam = spam[sample(1:n),]

# Affiche les données
dim(spam)
names(spam)

# Ensembles d'entrainement / de test
n.train = floor(0.8*n)
spam.train = spam[1:n.train,]
spam.test = spam[-(1:n.train),]

# Arbre de classification
library("tree")

spam.tree <- tree(spam ~ ., data=spam.train)
plot(spam.tree)
text(spam.tree)


errors = function(arbre, display = TRUE) {
  pred = predict(arbre, spam.test, type="class")
  faux_neg = sum(pred == "email" & spam.test$spam == "spam")
  faux_pos = sum(pred == "spam" & spam.test$spam == "email")
  if (display) {
    cat(faux_pos, 'faux positifs et ', faux_neg, 'faux negatifs \n')
    cat(faux_pos + faux_neg, 'erreurs')  
  } else {
    faux_pos + faux_neg
  }
}
errors(spam.tree)


# Un arbre grandit au maximum qui surapprends les données
spam.tree1 <- tree(spam ~ ., data=spam.train, 
                  control = tree.control(nobs = n.train, mindev = 0, minsize = 2))
plot(spam.tree1)
errors(spam.tree1)

# On élague cet arbre trop grand
spam.tree2 = prune.tree(spam.tree1, best = 50)
plot(spam.tree2)
errors(spam.tree2)

# À quel point faut-il élaguer ?
err = NULL
nnodes = seq(4,200,by = 3)
for (i in nnodes) {
  T = prune.tree(spam.tree1, best = i)
  err = c(err,errors(T, display = F))
}
plot(nnodes, err, type='l')

# Un critère de validation croisée
plot(cv.tree(spam.tree1, K = 10, method="misclass"))


# Forêts aléatoires.
library("randomForest")
spam.rf <- randomForest(spam ~ ., data=spam.train, ntree = 200, 
                        importance=TRUE, proximity=TRUE)
print(spam.rf)

errors(spam.rf)
plot(spam.rf$err.rate[,1], type='l', xlab = "nombre d'arbres", ylab = "erreur OOB")

# Afficher l'importance des variables
varImpPlot(spam.rf, type = 1, main = "Importance des variables pour la détection de spams")
